// ----------------------------------------------------------------
//  NECタイプの赤外線リモコン受信 2016/12/25  OK
//  Notes: 4M Hz 内部クロック
//     LED1 RC0,  LED2 RC1,  LED3 RC2,  LED4 RC3
//     Sencer  RB7
//    Language: MPLABX XC8
//    Target:   PIC16F1459
// ----------------------------------------------------------------
#define _XTAL_FREQ  4000000   // delay_ms(x) のための定義
#define L_ON_Min    5000      // 5msec リーダーの最小継続時間
#define DataTH      1000      // 1msec データの「１」「０」境界時間
#define D_Off_Max   2000      // 2msec データ「０」の最大継続時間
#define IrIN        RB7       // 赤外線センサー入力
#define LED1        LATC0
#define LED2        LATC1
#define LED3        LATC2
#define LED4        LATC3

#include <xc.h>

#pragma config FOSC  = INTOSC, WDTE = OFF, PWRTE = OFF, MCLRE = OFF
#pragma config CP  = OFF, BOREN = ON, CLKOUTEN = OFF, IESO  = OFF
#pragma config FCMEN = OFF, WRT = OFF, CPUDIV = NOCLKDIV
#pragma config USBLSCLK = 48MHz, PLLMULT = 3x, PLLEN = ENABLED
#pragma config STVREN = ON, BORV = LO, LPBOR = OFF, LVP = OFF

unsigned int IrCmdRcv(void);
unsigned int IrByteRcv(void);

// MAIN ********************************************************
void main(void){
    unsigned int revdData;
    OSCCON = 0b00110111;        // 内部クロック4Mhz
    LATC   = 0;                 // LED 初期化
    TRISC&=0xF0;
                                // Pin 状態変化 初期化 
    IOCBNbits.IOCBN7 = 1;       // 負極性の変異でWake-up
    INTCONbits.IOCIE = 1;       // Wake-up 許可

    // Timer 1 設定 -------------------------------------------
    // リモコンパルス幅の計測に使用する Timer は、1カウント 1uS
    T1CON  = 0b00000001;        // Timer 1 FOSC/4 1/1(66mSecごと)
    T1GCON = 0b00000000;        // Timer 1 Gate設定(使用せず)

    while(1){                   // 繰り返しループ
        IOCBFbits.IOCBF7 = 0;                   // 入力変位フラッグ クリア
        SLEEP();                                // 無信号時はスリープ
        //--------------------
        revdData = IrCmdRcv();                  // コマンド取得
        if(revdData == 0) continue;             // コマンド無
        // ---- 受信コマンドの処理 -----------------------------------
        switch (revdData){
            case 0x0321: LED1 = !LED1; break;   // Key 1 LED1 反転
            case 0x0322: LED2 = !LED2; break;   // Key 2 LED2 反転
            case 0x0323: LED3 = !LED3; break;   // Key 3 LED3 反転
            case 0x0324: LED4 = !LED4; break;   // Key 4 LED4 反転
            default:     LATC = 0;     break;   // 他のKey 全消灯
        }
    }
}

//*************** 赤外線コマンドを受信する *****************************
// 正常にコマンドを受信したときは、そのコマンドを返り値とし
// 無信号、エラー時は、0を返す。
// ビクターの信号体系（NEC系）を受信
// リーダ受信確認、カスタムコード(8bits)確認、コマンド受信(8bits)する
//*******************************************************************
unsigned int IrCmdRcv(void){
    unsigned int rcvData;
    unsigned int CmdRcv;
    if(IrIN) return 0;                    // 無信号「１」時は復帰

    // ------- Start Bit （リーダー）の確認 -----------------------------
    //OnおよびOFFの時間を確認しノイズか有効データか判断する
    // ---------------------------------------------------------------
    TMR1 = 0;                            // -- リーダーのON 時間測定
    while(TMR1 < L_ON_Min){              // リーダーの最小継続に達しないのに
        if(IrIN)return 0;                // IR Space「１」無信号になったら
    }                                    // ノイズと判断し復帰
    while(!IrIN);                        // IR Space 待ち
    TMR1 = 0;                            // -- リーダーのOFF時間測定
    while(IrIN){                         // リーダーの終了時間を越えても待ち
        if(TMR1 > L_ON_Min)return 0;     // data開始(IrON)しなければ復帰
    }
    // ------- カスタムコードの確認 -----------------------------------
    rcvData=IrByteRcv();                      //８ビット受信
    if((rcvData & 0xFF00) != 0x0800)return 0; //受信ビット数確認
    CmdRcv = rcvData << 8;
    // ------- コマンドコードの確認 -----------------------------------
    rcvData=IrByteRcv();
    if((rcvData & 0xFF00) != 0x0800)return 0;
    return (CmdRcv | (rcvData & 0xFF));
}

//****** データ Byteを受信する ***************************************
//  ８ビット受信する。
//  受信ビット数を  上位８ビットに
//  受信データを    下位８ビットに組み合わせて  戻る
//         「1」「０」判定は「０」の時間を DataTH と比較する
//  Stop ビット受信時は、その時点で復帰する
//*****************************************************************
unsigned int IrByteRcv(void){
    char data = 0;
    char i;
    for(i = 0; i < 8; i++){         // １ビットずつ８回繰り返す
        data >>= 1;
        while(!IrIN);                    // space 開始を待つ
        TMR1 = 0;                        //    Bitのspace時間測定開始
        while(IrIN)                      // space 終了を待つ
            if(TMR1 > D_Off_Max)         //    2mS以上ならStopなので
            return (int)i * 256 + data;  //    復帰
        if(TMR1 > DataTH)                // 時間取得「1」「０」判定
            data |= 0x80;                // 「1」を立てる
    }
    return (int)i * 256 + data;          // 受信Bit数と内容合成
}
